//----------------------------------------------------------------------------------------
// patterns & practices - Smart Client Software Factory - Guidance Package
//
// This file was generated by the "Add View" recipe.
//
// This class is the concrete implementation of a View in the Model-View-Presenter 
// pattern. Communication between the Presenter and this class is acheived through 
// an interface to facilitate separation and testability.
// Note that the Presenter generated by the same recipe, will automatically be created
// by CAB through [CreateNew] and bidirectional references will be added.
//
// For more information see:
// ms-help://MS.VSCC.v80/MS.VSIPCC.v80/ms.scsf.2006jun/SCSF/html/03-030-Model%20View%20Presenter%20%20MVP%20.htm
//
// Latest version of this Guidance Package: http://go.microsoft.com/fwlink/?LinkId=62182
//----------------------------------------------------------------------------------------

using System;
using System.Windows.Forms;
using Microsoft.Practices.CompositeUI.SmartParts;
using Microsoft.Practices.ObjectBuilder;
using TaskViewer.Infrastructure.Interface;
using TaskViewer.Infrastructure.Interface.BusinessEntities;
using Microsoft.Practices.CompositeUI.Utility;

using TaskViewer.Infrastructure.Library.UI;

namespace TaskViewer.AdministrationModule
{
    [SmartPart]
    public partial class UserDetails : UserControl, IUserDetails, ISmartPartInfoProvider
    {
        User _user;
        Boolean _editState = false;
        Boolean _createMode = false;

        public UserDetails()
        {
            InitializeComponent();

            ((IUserDetails)this).ResetViewToEmpty();
        }

        private void ChangeState(Boolean state)
        {
            _editState = state;
            _userName.Enabled = state;
            _userFullName.Enabled = state;
            _userEmail.Enabled = state;
            _cbAdministrator.Enabled = state;
            _cbAccount.Enabled = state;
            _confirmPassword.Enabled = state;
            _password.Enabled = state;
            _ok.Visible = state;
            if (state)
            {
                _cancel.Text = Properties.Resources.btnCancelText;
            }
            else
            {
                _cancel.Text = Properties.Resources.btnEditText;
            }

        }

        /// <summary>
        /// Sets the presenter. The dependency injection system will automatically
        /// create a new presenter for you.
        /// </summary>
        [CreateNew]
        public UserDetailsPresenter Presenter
        {
            set
            {
                _presenter = value;
                _presenter.View = this;
            }
        }

        protected override void OnLoad(EventArgs e)
        {
            _presenter.OnViewReady();
        }

        private void listBox1_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {

        }

        private void label4_Click(object sender, EventArgs e)
        {

        }

        private void button1_Click(object sender, EventArgs e)
        {

        }
      
        private delegate void ResetViewToEmptyDelegate();

        public void ResetViewToEmpty()
        {
            if (InvokeRequired)
            {
                BeginInvoke(new ResetViewToEmptyDelegate(InnerResetViewToEmpty));
            }
            else
            {
                InnerResetViewToEmpty();
            }
        }



        private void InnerResetViewToEmpty()
        {
            _user = null;
            _userDetailsPanel.BringToFront();
        }

        private delegate void ShowUserDelegate(User user);
        public void ShowUser(User user)
        {

            Guard.ArgumentNotNull(user, "user");
            if (InvokeRequired)
            {
                BeginInvoke(new ShowUserDelegate(InnerShowUser));
            }
            else
            {
                InnerShowUser(user);
            }

        }

        private void InnerShowUser(User user)
        {
            _user = user;
            _userDetailsPanel.Hide();
            ChangeState(false);
            _userName.Text = _user.UserName;
            _userFullName.Text = _user.UserFullName;
            _userEmail.Text = _user.UserEmail ;
            _cbAccount.Checked = _user.IsAccountLocked;
            _cbAdministrator.Checked = _user.IsAdministrator;
            

        }

        private User UpdateUser()
        {
            User user = new User(
                _user.DateCreated,
                _cbAccount.Checked,
                _cbAdministrator.Checked,
                _userEmail.Text,
                _userFullName.Text,
                _user.UserID,
                _userName.Text
                );

            return user;


        }

        private void _edit_Click(object sender, EventArgs e)
        {
            ChangeState(!_editState);
        }

        private void _save_Click(object sender, EventArgs e)
        {
            ChangeState(!_editState);
            _presenter.UpdateUser(this.UpdateUser());

        }


        public bool isCreateMode
        {
            get
            {
                return _createMode;
            }
            set
            {
                _createMode = value;
                if (value)
                {
                    _userDetailsPanel.Hide();
                    _editState = true;
                }
            }
        }


        #region ISmartPartInfoProvider Members

        public ISmartPartInfo GetSmartPartInfo(Type smartPartInfoType)
        {
            ISmartPartInfo spi;


            if (smartPartInfoType.IsAssignableFrom(typeof(WindowSmartPartInfo)))
            {
                WindowSmartPartInfo wspi = new WindowSmartPartInfo();
                wspi.Modal = true;
                wspi.MinimizeBox = false;
                wspi.MaximizeBox = false;
                wspi.ControlBox = true;

                wspi.Keys[WindowWorkspaceSetting.FormBorderStyle] = FormBorderStyle.FixedDialog;
                wspi.Keys[WindowWorkspaceSetting.AcceptButton] = _ok;
                wspi.Keys[WindowWorkspaceSetting.CancelButton] = _cancel;
                spi = wspi;
            }
            else
            {
                spi = Activator.CreateInstance(smartPartInfoType) as ISmartPartInfo;
            }

            spi.Description = Properties.Resources.NewUserDescription;
            spi.Title = Properties.Resources.NewUserTitle;

            return spi;
        }

        #endregion
    }
}

