using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Globalization;

namespace TaskViewer.Infrastructure.Interface.Services
{
    public class ShellNotificationService : IMessageBoxService
    {
        private Control _owner;

        public ShellNotificationService()
            : this(null)
        {
        }

        public ShellNotificationService(Control owner)
        {
            _owner = owner;
        }

        private bool GetIsRtl(Control control)
        {
            if (control == null)
                return CultureInfo.CurrentUICulture.TextInfo.IsRightToLeft;

            if (control.RightToLeft == RightToLeft.Yes)
            {
                return true;
            }
            else if (control.RightToLeft == RightToLeft.No)
            {
                return false;
            }
            else if (control.RightToLeft == RightToLeft.Inherit)
            {
                if (control.Parent == null)
                {
                    throw new InvalidOperationException();
                }
                return GetIsRtl(control.Parent);
            }
            return false;
        }

        private bool IsRtl
        {
            get { return GetIsRtl(_owner); }
        }

        public DialogResult Show(string text)
        {
            return Show(text, null, MessageBoxButtons.OK, MessageBoxIcon.None);
        }

        public DialogResult Show(string text, string caption)
        {
            return Show(text, caption, MessageBoxButtons.OK, MessageBoxIcon.None);
        }

        public DialogResult Show(string text, string caption, MessageBoxButtons buttons)
        {
            return Show(text, caption, buttons, MessageBoxIcon.None);
        }

        private delegate DialogResult ShowDelegate(string text, string caption, MessageBoxButtons buttons, MessageBoxIcon icon
            );

        public DialogResult Show(string text, string caption, MessageBoxButtons buttons, MessageBoxIcon icon)
        {
            if (_owner != null && _owner.InvokeRequired)
            {
                return (DialogResult)_owner.Invoke(new ShowDelegate(Show), text, caption, buttons, icon);
            }
            else
            {
                MessageBoxOptions messageBoxOptions = (MessageBoxOptions)0;
                if (IsRtl)
                {
                    messageBoxOptions |= MessageBoxOptions.RightAlign | MessageBoxOptions.RtlReading;
                }
                return MessageBox.Show(_owner,
                                           text, caption, buttons, icon,
                                           MessageBoxDefaultButton.Button1,
                                           messageBoxOptions);

            }
        }
    }
}
