using System;
using System.Windows.Forms;
using Microsoft.Practices.ObjectBuilder;
using TaskViewer.Infrastructure.Interface.Constants;
using Microsoft.Practices.CompositeUI.Commands;

namespace TaskViewer.Infrastructure.Layout
{
    public partial class ShellLayoutView : UserControl
    {
        private ShellLayoutViewPresenter _presenter;

        //internal ToolStrip NavigationToolStrip
        //{
        //    get
        //    {
        //        return this._navigationStrip;
        //    }
            

        //}

        /// <summary>
        /// Initializes a new instance of the <see cref="T:ShellLayoutView"/> class.
        /// </summary>
        public ShellLayoutView()
        {
            InitializeComponent();
            _navigationWorkspace.Name = WorkspaceNames.NavigationWorkspace;
            _summaryWorkspace.Name = WorkspaceNames.SummaryWorkspace;
            _detailWorkspace.Name = WorkspaceNames.DetailWorkspace;
        }

        /// <summary>
        /// Sets the presenter.
        /// </summary>
        /// <value>The presenter.</value>
        [CreateNew]
        public ShellLayoutViewPresenter Presenter
        {
            set
            {
                _presenter = value;
                _presenter.View = this;
            }
        }

        /// <summary>
        /// Gets the main menu strip.
        /// </summary>
        /// <value>The main menu strip.</value>
        internal MenuStrip MainMenuStrip
        {
            get { return _mainMenuStrip; }
        }

        /// <summary>
        /// Gets the main status strip.
        /// </summary>
        /// <value>The main status strip.</value>
        internal StatusStrip MainStatusStrip
        {
            get { return _mainStatusStrip; }
        }

        /// <summary>
        /// Gets the main toolbar strip.
        /// </summary>
        /// <value>The main toolbar strip.</value>
        internal ToolStrip MainToolbarStrip
        {
            get { return _mainToolStrip; }
        }

        /// <summary>
        /// Gets the main toolbar strip.
        /// </summary>
        /// <value>The main toolbar strip.</value>
        internal ToolStrip NavigationToolStrip
        {
            get { return _navigationToolStrip; }
        }

        /// <summary>
        /// Close the application.
        /// </summary>
        private void OnFileExit(object sender, EventArgs e)
        {
            _presenter.OnFileExit();
        }

        /// <summary>
        /// Sets the status label.
        /// </summary>
        /// <param name="text">The text.</param>
        public void SetStatusLabel(string text)
        {
            _offlineStatus.Text = text;
        }

        public Microsoft.Practices.CompositeUI.SmartParts.ISmartPartInfo GetSmartPartInfo(Type smartPartInfoType)
        {
            // Implement ISmartPartInfoProvider in the containing smart part. Required in order for contained smart part infos to work.
            //Microsoft.Practices.CompositeUI.SmartParts.ISmartPartInfoProvider ensureProvider = this;
            return this.infoProvider.GetSmartPartInfo(smartPartInfoType);

        }

        [CommandHandler(CommandNames.ChangeOfflineImage)]
        public void OnChangeOfflineImage(object sender, EventArgs args)
        {
            _offlineStatus.Text = Properties.Resources.OnlineWork;
            _offlineStatus.Image = Properties.Resources.StatusConnected.ToBitmap();
        }

        [CommandHandler(CommandNames.ChangeOnlineImage)]
        public void OnChangeOnlineImage(object sender, EventArgs args)
        {
            _offlineStatus.Text = Properties.Resources.OfflineWork;
            _offlineStatus.Image = Properties.Resources.StatusDisconnected.ToBitmap();
        }
    }
}
