//----------------------------------------------------------------------------------------
// patterns & practices - Smart Client Software Factory - Guidance Package
//
// This file was generated by the "Add View" recipe.
//
// A presenter calls methods of a view to update the information that the view displays. 
// The view exposes its methods through an interface definition, and the presenter contains
// a reference to the view interface. This allows you to test the presenter with different 
// implementations of a view (for example, a mock view).
//
// For more information see:
// ms-help://MS.VSCC.v80/MS.VSIPCC.v80/ms.scsf.2006jun/SCSF/html/03-030-Model%20View%20Presenter%20%20MVP%20.htm
//
// Latest version of this Guidance Package: http://go.microsoft.com/fwlink/?LinkId=62182
//----------------------------------------------------------------------------------------

using System;
using System.Collections.Specialized;
using TaskViewer.Infrastructure.Interface.BusinessEntities;
using TaskViewer.Infrastructure.Module.Services;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeUI;

using TaskViewer.Infrastructure.Interface;
using TaskViewer.Infrastructure.Library.AuthorizationService;
using Microsoft.Practices.CompositeUI.Services;
using Microsoft.Practices.CompositeUI.Configuration;
using TaskViewer.Infrastructure.Interface.Services;
using TaskViewer.Infrastructure.Library.Services;

namespace TaskViewer.Infrastructure.Module
{
    public class AuthenticationViewPresenter : Presenter<IAuthenticationView>
    {
        private IAuthorizationService _authorizationService;
        private string _authorizationTicket;
        private UserInformation _userInformation;
        private IModuleLoaderService _moduleLoader;
        private FileCatalogModuleEnumerator _moduleEnumerator;
        private WorkItem _workItem;


        [InjectionConstructor]
        public AuthenticationViewPresenter
			(
            [ServiceDependency] IAuthorizationService authorizationService,
            [ServiceDependency] IModuleLoaderService loader,
            [ServiceDependency] FileCatalogModuleEnumerator modEnumerator,
            [ServiceDependency] WorkItem workItem
			)
        {
            _authorizationService = authorizationService;
            _moduleEnumerator = modEnumerator;
            _moduleLoader = loader;
            _workItem = workItem;
        }
        /// <summary>
        /// This method is a placeholder that will be called by the view when it's been loaded <see cref="System.Winforms.Control.OnLoad"/>
        /// </summary>
        public override void OnViewReady()
        {
            base.OnViewReady();
        }

        public void AuthorizeUser(string username, string password)
        {
            try
            {
                _authorizationTicket = _authorizationService.GetAuthorizationTicket(username, password);
                if (_authorizationTicket == "-1")
                {
                    View.ShowError("User not authorized. Check your username and password.");
                    return;
                }
                _userInformation = _authorizationService.GetUserInfo(_authorizationTicket);
            }
            catch(Exception ex)
            {
                View.ShowError(ex.Message);
            }

            
            NameValueCollection settings = new System.Collections.Specialized.NameValueCollection();
            settings.Add(UserFields.UserName.ToString(), _userInformation.userName);
            if (_userInformation.isAdministrator)
            {
                settings.Add(UserFields.Role.ToString(), UserRoles.Administrator.ToString());
            }
            else
            {
                settings.Add(UserFields.Role.ToString(), UserRoles.User.ToString());
            }
            WorkItem.Services.Remove<IAuthenticationService>();
           
            IAuthenticationService authService = WorkItem.Services.AddNew<SimpleWinFormAuthenticationService, IAuthenticationService>();

            ((IConfigurable)authService).Configure(settings);
            WorkItem.Services.AddNew<GenericPrincipalImpersonationService, IImpersonationService>();
            IImpersonationService impersonator = WorkItem.Services.Get<IImpersonationService>();
            
            using (IImpersonationContext context = impersonator.Impersonate())
            {
                IModuleInfo[] modules = _moduleEnumerator.EnumerateModules();
                foreach(IModuleInfo info in modules)
                {
                    _moduleLoader.Load(_workItem, info);
                }

            }
            CloseView();
        }

        /// <summary>
        /// Close the view
        /// </summary>
        public void OnCloseView()
        {
            base.CloseView();
        }
    }
}

