using System.Threading;
using Microsoft.Practices.CompositeUI;
using Microsoft.Practices.CompositeUI.Services;
using Microsoft.Practices.ObjectBuilder;
using TaskViewer.Infrastructure.Interface.BusinessEntities;
using TaskViewer.Infrastructure.Interface.Services;

namespace TaskViewer.Infrastructure.Module.Services
{
    public class GenericPrincipalImpersonationService : IImpersonationService, IConfigurable
    {
        private IAuthenticationService _authService = null;
        [InjectionConstructor]
        public GenericPrincipalImpersonationService(
                                    [ServiceDependency] IAuthenticationService authService
                                    
                                    )
        {
            _authService = authService;
        }

        public IAuthenticationService AuthenticationService
        {
            get { return _authService; }
        }

        public IImpersonationContext Impersonate()
        {
            IImpersonationContext context = new GenericPrincipalImpersonationContext();
            context.State = Thread.CurrentPrincipal;

            // the AuthenticationService sets the new principal
            _authService.Authenticate();

            return context;
        }

        #region IConfigurable Members

        public void Configure(System.Collections.Specialized.NameValueCollection settings)
        {
            IConfigurable configurableService = _authService as IConfigurable;
            if (configurableService != null)
            {
                configurableService.Configure(settings);
            }
        }

        #endregion
    }
}
