using System;
using System.Collections.Generic;
using System.Text;


using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeUI;
using Microsoft.Practices.EnterpriseLibrary.Caching;
using Microsoft.Practices.CompositeUI.EventBroker;

using TaskViewer.TaskManagement.Interface.Constants;
using TaskViewer.Infrastructure.Interface.BusinessEntities;
using TaskViewer.Infrastructure.Interface.Services;
using TaskViewer.TaskManagement.ServiceAgent.BusinessEntities;

using TaskManagement.ServiceProxies.TaskViewer.DataService;
using TaskManagement.ServiceProxies.TaskViewer.AuthService;
using TaskViewer.TaskManagement.ServiceAgent;
using TaskViewer.TaskManagement.ServiceAgent.EntityTranslator;

//TODO Check the dispose issue see Appraisals Agent
namespace TaskViewer.TaskManagement.ServiceAgent
{
    public class TaskManagementServiceAgent: ITaskManagementServiceAgent
    {
        private IDataServiceProxy _dataServiceProxy;
        private IAuthorizationService _authServiceProxy;

        private Boolean _isOfflineWork;
        private TasksInfo _dsTasks = new TasksInfo();
        private ProjectsInfo _dsProjects = new ProjectsInfo();
        private LookupInfo _dsLookupTables = new LookupInfo();
        private List<Priority> _priorities =  new List<Priority>();
        private List<Status> _statuses = new List<Status>();
        private List<User> _users = new List<User>();
        private List<Project> _projects = new List<Project>();
        private Dictionary<int, List<Task>> _projectTasks = new Dictionary<int, List<Task>>(); 
        private User _currentUserInformation = new User();
        private string _ticket = String.Empty;

        public string Ticket
        {
            get
            {
                if (string.IsNullOrEmpty(_ticket))
                {
                   // _ticket = _authServiceProxy.GetAuthorizationTicket("username", "pass");
                }

                return _ticket;
            }
            set { _ticket = value; }
        }
        private CacheManager _taskCache;

        private IEntityTranslatorService _translator = null;

        [ServiceDependency]
        public IEntityTranslatorService Translator
        {
            get { return _translator; }
            set { _translator = value; }
        }



        
        [InjectionConstructor]
        public TaskManagementServiceAgent()
            : this(new DataServiceProxy(), new AuthorizationService())
        {
            if (_isOfflineWork)
            {
                LoadOfflineInfo();
            }
            else
            {
                //TODO add logic for feeling name and password
               
                _dataServiceProxy.GetLookupTables
                    (Settings.Default.LookUpTimeout,
                    Ticket,
                    GetLookupTableCallBack);




            }
        }

        internal void GetLookupTableCallBack(bool success, LookupInfo lookUpInfo)
        {
            if (success)
            {
                _dsLookupTables = lookUpInfo;
                
                _priorities.Clear();
                _users.Clear();
                _statuses.Clear();
                foreach (LookupInfo.PrioritiesRow priorityRow in _dsLookupTables.Priorities.Rows)
                {
                    _priorities.Add(Translator.Translate<Priority>(priorityRow));
                }
                foreach (LookupInfo.StatusesRow statusRow in _dsLookupTables.Statuses.Rows)
                {
                    _statuses.Add(Translator.Translate<Status>(statusRow));
                }
                foreach (LookupInfo.UsersRow userRow in _dsLookupTables.Users.Rows)
                {
                   _users.Add(Translator.Translate<User>(userRow));
                }
            }
            else
            {
                throw new NotImplementedException();
            }
        }

        private void LoadOfflineInfo()
        {
            //TODO
            throw new Exception("The method or operation is not implemented.");
        }

        public TaskManagementServiceAgent(IDataServiceProxy dataServiceProxy, IAuthorizationService authServiceProxy)
        {
            _dataServiceProxy = dataServiceProxy;
            _authServiceProxy = authServiceProxy;
        }

        #region Data Service
        //public DataLayerResult GetProjects()
        //{
        //    DataSetProjects ds;

        //    try
        //    {
        //        ds = m_WsData.GetProjects(m_Ticket);

        //        //' All TaskVision web services return nothing to indicate an expired ticket
        //        if (ds == null)
        //        {
        //            // Get a new ticket and try the call again
        //            DataLayerResult ticketResult = GetAuthorizationTicket();
        //            if (ticketResult != DataLayerResult.Success)
        //                return ticketResult;

        //            ds = m_WsData.GetProjects(m_Ticket);

        //            // This next block should never happen. It means it took more than TIMEOUT 
        //            // (default is 2 min) between GetAuthTicket and GetProjects
        //            if (ds == null)
        //                return DataLayerResult.AuthenticationFailure;
        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        return HandleException(ex);
        //    }

        //    DsProjects.Clear();
        //    DsProjects.Merge(ds);
        //    return DataLayerResult.Success;
        //}


        //public DataLayerResult GetTasks(int projectID, bool clearData)
        //{
        //    DataSetTasks ds;

        //    try
        //    {
        //        ds = m_WsData.GetTasks(m_Ticket, projectID);

        //        if (ds == null)
        //        {
        //            DataLayerResult ticketResult = GetAuthorizationTicket();
        //            if (ticketResult != DataLayerResult.Success)
        //                return ticketResult;

        //            ds = m_WsData.GetTasks(m_Ticket, projectID);

        //            if (ds == null)
        //                return DataLayerResult.AuthenticationFailure;

        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        return HandleException(ex);
        //    }

        //    if (clearData)
        //        DsTasks.Tasks.Clear();

        //    DsTasks.Merge(ds);
        //    return DataLayerResult.Success;
        //}

        //public DataLayerResult GetLookUpTables()
        //{
        //    DataSetLookupTables ds;

        //    try
        //    {
        //        ds = m_WsData.GetLookupTables(m_Ticket);

        //        if (ds == null)
        //        {
        //            DataLayerResult ticketResult = GetAuthorizationTicket();
        //            if (ticketResult != DataLayerResult.Success)
        //                return ticketResult;

        //            ds = m_WsData.GetLookupTables(m_Ticket);

        //            if (ds == null)
        //                return DataLayerResult.AuthenticationFailure;
        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        return HandleException(ex);
        //    }

        //    DsLookupTables.Priorities.Clear();
        //    DsLookupTables.Users.Clear();
        //    DsLookupTables.Statuses.Clear();
        //    DsLookupTables.Merge(ds);

        //    return DataLayerResult.Success;
        //}

        //public DataLayerResult InsertProject(ref int projectID, string projectName, string projectDescription)
        //{
        //    try
        //    {
        //        // note: projectID is ByRef
        //        projectID = m_WsData.InsertProject(m_Ticket, projectName, projectDescription);

        //        // -1 is our magic number for nothing, like above the ws returns nothing if the ticket is expired
        //        if (projectID == -1)
        //        {
        //            DataLayerResult ticketResult = GetAuthorizationTicket();
        //            if (ticketResult != DataLayerResult.Success)
        //                return ticketResult;

        //            projectID = m_WsData.InsertProject(m_Ticket, projectName, projectDescription);

        //            if (projectID == -1)
        //                return DataLayerResult.AuthenticationFailure;

        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        return HandleException(ex);
        //    }


        //    return DataLayerResult.Success;
        //}

        //public DataLayerResult UpdateTasks(int projectID)
        //{
        //    DataSetTasks ds;

        //    try
        //    {
        //        // note that we are sending the whole dataset
        //        // if during data collision handling, the web service breaks
        //        // we'll be stuck with a short dataset
        //        ds = m_WsData.UpdateTasks(m_Ticket, projectID, DsTasks);

        //        if (ds == null)
        //        {
        //            DataLayerResult ticketResult = GetAuthorizationTicket();
        //            if (ticketResult != DataLayerResult.Success)
        //                return ticketResult;

        //            ds = m_WsData.UpdateTasks(m_Ticket, projectID, DsTasks);

        //            if (ds == null)
        //                return DataLayerResult.AuthenticationFailure;
        //        }
        //    }
        //    catch (Exception ex)
        //    {
        //        return HandleException(ex);
        //    }

        //    DsTasks.Tasks.Clear();
        //    DsTasks.Merge(ds);

        //    return DataLayerResult.Success;
        //}

        //public IAsyncResult BeginGetTasks(int projectID)
        //{
        //    try
        //    {
        //        // workaround for the expiring ticket during idle time
        //        // start an async call for ticket
        //        IAsyncResult authAr = m_WsAuth.BeginGetAuthorizationTicket(CurrentUserInformation.UserName,
        //            CurrentUserInformation.UserPassword, null, null);

        //        // start an async call for the dataset and send the ticket async result along for the ride
        //        return m_WsData.BeginGetTasks(m_Ticket, projectID, null, authAr);
        //    }
        //    catch (Exception ex)
        //    {
        //        LogError.Write(ex.Message + System.Environment.NewLine + ex.StackTrace);
        //        return null;
        //    }
        //}

        //public DataLayerResult EndGetTasks(IAsyncResult ar)
        //{
        //    DataSetTasks ds;

        //    try
        //    {
        //        // this method was called after checking that both results are completed
        //        // grab the new ticket
        //        m_Ticket = m_WsAuth.EndGetAuthorizationTicket((IAsyncResult)ar.AsyncState);

        //        // grab the new dataset
        //        ds = m_WsData.EndGetTasks(ar);

        //        if (ds == null)
        //            return DataLayerResult.AuthenticationFailure;
        //    }
        //    catch (Exception ex)
        //    {
        //        return HandleException(ex);
        //    }

        //    DsTasks.Tasks.Clear();
        //    DsTasks.Merge(ds);

        //    return DataLayerResult.Success;
        //}

        
        //#endregion

        //#region Helper Functions
        //private DataLayerResult HandleException(Exception ex)
        //{
        //    LogError.Write(ex.Message + System.Environment.NewLine + ex.StackTrace);

        //    if (ex is WebException)
        //        return DataLayerResult.ConnectionFailure;
        //    else if (ex is SoapException)
        //        return DataLayerResult.ServiceFailure;
        //    else
        //        return DataLayerResult.UnknownFailure;
        //}
        #endregion

        #region Events
        [EventPublication(EventTopicNames.ReferenceInfoLoaded, PublicationScope.Global)]
        public event EventHandler ReferenceInfoLoaded;
        #endregion

        #region ITaskManagementServiceAgent Members

        public void GetProjects(GetProjectsCallback callback)
        {
            if(_projects.Count != 0)
            {
                callback(_projects.ToArray());
            }
            else
            {
                _dataServiceProxy.GetProjects(
                   Settings.Default.ProjectTimeout,
                   Ticket,
                   delegate(bool success, ProjectsInfo info)
                   {
                       if(success)
                       {
                           _dsProjects = info;
                           foreach(ProjectsInfo.ProjectsRow project in info.Projects.Rows)
                           {
                               _projects.Add(Translator.Translate<Project>(project));
                           }
                           
                           callback(_projects.ToArray());
                       }
                       else
                       {
                           throw new NotImplementedException();
                       }
                   });
                
            }
        }


        public void GetTasks(Int32 projectID, GetTasksCallback callback)
        {
            if (!_projectTasks.ContainsKey(projectID))
            {
                _dataServiceProxy.GetTasks(
                    Settings.Default.TaskTimeout,
                    Ticket,
                    projectID,
                    true,
                    delegate(bool success, TasksInfo info)
                    {
                        if (success)
                        {
                            List<Task> tasks = new List<Task>();
                            foreach (TasksInfo.TasksRow row in info.Tasks.Rows)
                            {
                                tasks.Add(Translator.Translate<Task>(row));
                            }
                            _projectTasks.Add(projectID, tasks);
                            callback(tasks.ToArray());
                            if (ReferenceInfoLoaded != null)
                            {
                                ReferenceInfoLoaded(this, null);
                            }
                        }
                        else
                        {
                            throw new NotImplementedException();
                        }
                    });

            }
            else
            {
                callback(_projectTasks[projectID].ToArray());
            }

        }


        #region Referenced information
        public TaskManagement.ServiceAgent.BusinessEntities.Priority[] GetPriorities()
        {

            //TODO add copy not return actual reference
            //implement deep copy
            return _priorities.ToArray();
            
        }

        public TaskManagement.ServiceAgent.BusinessEntities.Status[] GetStatuses()
        {
            
            //TODO add copy not return actual reference
            //implement deep copy
            return _statuses.ToArray();
        }

        public User[] GetUsers()
        {
            //TODO add copy not return actual reference
            //implement deep copy
            return _users.ToArray();
        }
        #endregion 

        public void AddProject()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void AddTask()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void UpdateTask()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void GoOnline()
        {
            _isOfflineWork = false;
            //synchronize
            throw new Exception("The method or operation is not implemented.");
        }

        public void GoOffline()
        {
            _isOfflineWork = true;
            //cache all the necessary information
            throw new Exception("The method or operation is not implemented.");
        }

        
        //TODO add calllback for updating UI
        public void UpdateTask(Task task)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion

        
        internal void RetrieveTasksCallback(bool success, TasksInfo tasksInfo)
        {
            if (success)
            {
               // _dsTasks = tasksInfo;

            }
            else
            {
            }

        }
    }
}
