//----------------------------------------------------------------------------------------
// patterns & practices - Smart Client Software Factory - Guidance Package
//
// This file was generated by the "Add View" recipe.
//
// This class is the concrete implementation of a View in the Model-View-Presenter 
// pattern. Communication between the Presenter and this class is acheived through 
// an interface to facilitate separation and testability.
// Note that the Presenter generated by the same recipe, will automatically be created
// by CAB through [CreateNew] and bidirectional references will be added.
//
// For more information see:
// ms-help://MS.VSCC.v80/MS.VSIPCC.v80/ms.scsf.2006jun/SCSF/html/03-030-Model%20View%20Presenter%20%20MVP%20.htm
//
// Latest version of this Guidance Package: http://go.microsoft.com/fwlink/?LinkId=62182
//----------------------------------------------------------------------------------------

using System;
using System.Windows.Forms;

using Microsoft.Practices.CompositeUI.SmartParts;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeUI.Utility;

using TaskViewer.Infrastructure.Interface;
using TaskViewer.TaskManagement.ServiceAgent.BusinessEntities;
using Microsoft.Practices.CompositeUI;
using TaskViewer.Infrastructure.Interface.Services;
using TaskViewer.Infrastructure.Library.UI;

namespace TaskViewer.TaskManagement
{
    [SmartPart]
    public partial class CollisionView : UserControl, ICollisionView, ISmartPartInfoProvider
    {
        public CollisionView()
        {
            InitializeComponent();
        }

        IMessageBoxService _messageBoxService;

        [ServiceDependency]
        public IMessageBoxService MessageBoxService
        {
            set { _messageBoxService = value; }
        }

        /// <summary>
        /// Sets the presenter. The dependency injection system will automatically
        /// create a new presenter for you.
        /// </summary>
        [CreateNew]
        public CollisionViewPresenter Presenter
        {
            set
            {
                _presenter = value;
                _presenter.View = this;
            }
        }

        protected override void OnLoad(EventArgs e)
        {
            _presenter.OnViewReady();
        }



        #region ICollisionView Members

        public void InnerShowCurrentTask(Task task)
        {
            tbCurrentAssignTo.Text = task.AssignedTo;
            tbCurrentDetails.Text = task.TaskDescription;
            tbCurrentDueDate.Text = task.DueDate.ToString();
            tbCurrentPriority.Text = task.Priority;
            tbCurrentProgress.Text = task.Progress.ToString();
            tbCurrentStatus.Text = task.Status;
            tbCurrentSummary.Text = task.TaskSummary;
            
        }

        public void InnerShowServerTask(Task task)
        {
            tbPreviousAssignTo.Text = task.AssignedTo;
            tbPreviousDetails.Text = task.TaskDescription;
            tbPreviousDueDate.Text = task.DueDate.ToString();
            tbPreviousPriority.Text = task.Priority;
            tbPreviousProgress.Text = task.Progress.ToString();
            tbPreviousSummary.Text = task.TaskSummary;
            tbPreviousStatus.Text = task.Status;
            lblPreviousMadeBy.Text = task.ModifiedBy;
            lblPreviousModifiedOn.Text = task.DateModified.ToString();
        }

        #endregion

        #region ICollisionView Members

        delegate void ShowTaskDelegate(Task task);

        public void ShowCurrentTask(Task task)
        {
            Guard.ArgumentNotNull(task, "task");
            if (InvokeRequired)
            {
                BeginInvoke(new ShowTaskDelegate(InnerShowCurrentTask));
            }
            else
            {
                InnerShowCurrentTask(task);
            }
        }

        public void ShowServerTask(Task task)
        {
            Guard.ArgumentNotNull(task, "task");
            if (InvokeRequired)
            {
                BeginInvoke(new ShowTaskDelegate(InnerShowServerTask));
            }
            else
            {
                InnerShowServerTask(task);
            }
        }

        #endregion

        #region ISmartPartInfoProvider Members

        public ISmartPartInfo GetSmartPartInfo(Type smartPartInfoType)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion

        #region ISmartPartInfoProvider Members

        ISmartPartInfo ISmartPartInfoProvider.GetSmartPartInfo(Type smartPartInfoType)
        {
            ISmartPartInfo spi;


            if (smartPartInfoType.IsAssignableFrom(typeof(WindowSmartPartInfo)))
            {
                WindowSmartPartInfo wspi = new WindowSmartPartInfo();
                wspi.Modal = true;
                wspi.MinimizeBox = false;
                wspi.MaximizeBox = false;
                wspi.ControlBox = true;

                wspi.Keys[WindowWorkspaceSetting.FormBorderStyle] = FormBorderStyle.FixedDialog;
                //TODO wspi.Keys[WindowWorkspaceSetting.AcceptButton] = ;
                //TODO wspi.Keys[WindowWorkspaceSetting.CancelButton] = _cancel;
                spi = wspi;
            }
            else
            {
                spi = Activator.CreateInstance(smartPartInfoType) as ISmartPartInfo;
            }

            spi.Description = Properties.Resources.CollisionDescription;
            spi.Title = Properties.Resources.CollisionTitle;

            return spi;
        }

        #endregion
    }
}

