//----------------------------------------------------------------------------------------
// patterns & practices - Smart Client Software Factory - Guidance Package
//
// This file was generated by the "Add Business Module" recipe.
//
// This class contains placeholder methods for the common module initialization 
// tasks, such as adding services, or user-interface element
//
// For more information see: 
// ms-help://MS.VSCC.v80/MS.VSIPCC.v80/ms.scsf.2006jun/SCSF/html/03-280-Automation%20Add%20Business%20Module.htm
//
// Latest version of this Guidance Package: http://go.microsoft.com/fwlink/?LinkId=62182
//----------------------------------------------------------------------------------------

using System;
using System.Windows.Forms;

using TaskViewer.Infrastructure.Interface;
using TaskViewer.Infrastructure.Interface.Services;
using TaskViewer.TaskManagement.Constants;
using TaskViewer.TaskManagement.ServiceAgent;
using TaskViewer.TaskManagement.ServiceAgent.EntityTranslator;

using Microsoft.Practices.CompositeUI;
using Microsoft.Practices.CompositeUI.Commands;
using System.Drawing;

namespace TaskViewer.TaskManagement
{
    public class ModuleController : WorkItemController
    {
        private ToolStripButton _offlineButton;
        private Boolean _isOffline = false;
        public override void Run()
        {
            AddServices();
            ExtendMenu();
            ExtendToolStrip();
            AddViews();
        }

        private void AddServices()
        {
            
            WorkItem.Services.AddNew<TaskManagementServiceAgent,ITaskManagementServiceAgent>();
            RegisterTranslators();
            //TODO: add services provided by the Module. See: Add or AddNew method in 
            //		WorkItem.Services collection or see ms-help://MS.VSCC.v80/MS.VSIPCC.v80/ms.practices.2005Nov.cab/CAB/html/03-020-Adding%20Services.htm
        }

        private void RegisterTranslators()
        {
            IEntityTranslatorService translator = WorkItem.Services.Get<IEntityTranslatorService>();
            translator.RegisterEntityTranslator(new PriorityTranslator());
            translator.RegisterEntityTranslator(new UserTranslator());
            translator.RegisterEntityTranslator(new StatusTranslator());
            translator.RegisterEntityTranslator(new ProjectTranslator());
            translator.RegisterEntityTranslator(new TaskTranslator());

        }

        private void ExtendMenu()
        {
            ToolStripMenuItem projectManagementMenuItem = new ToolStripMenuItem(Properties.Resources.ProjectMenuItem);
            WorkItem.UIExtensionSites[UIExtensionSiteNames.MainMenu].Add(projectManagementMenuItem);

            AddMenuItem(projectManagementMenuItem, Properties.Resources.NewProjectMenuItem, CommandNames.NewProject, Keys.P|Keys.Alt);

            AddMenuItem(projectManagementMenuItem, Properties.Resources.NewTaskMenuItem, CommandNames.NewTask, Keys.T | Keys.Alt);
            //TODO: add menu items here, normally by calling the "Add" method on
            //		on the WorkItem.UIExtensionSites collection. For an example 
            //		See: ms-help://MS.VSCC.v80/MS.VSIPCC.v80/ms.practices.2005Nov.cab/CAB/html/03-100-Showing%20UIElements.htm
        }

        private void AddMenuItem(ToolStripMenuItem parent, string text, string command, Keys shortcutKeys)
        {
            ToolStripMenuItem item = new ToolStripMenuItem(text);
            item.ShortcutKeys = shortcutKeys;
            parent.DropDownItems.Add(item);

            WorkItem.Commands[command].AddInvoker(item, "Click");
        }

        private void ExtendToolStrip()
        {
            _offlineButton = new ToolStripButton(Properties.Resources.WorkOffline, Properties.Resources.ToolWorkOffline.ToBitmap());
            WorkItem.UIExtensionSites[UIExtensionSiteNames.MainToolbar].Add(_offlineButton);
            WorkItem.Commands[CommandNames.OfflineSelection].AddInvoker(_offlineButton, "Click");

            ToolStripButton testbutton2 = new ToolStripButton("Collision");
            WorkItem.UIExtensionSites[UIExtensionSiteNames.MainToolbar].Add(testbutton2);

            WorkItem.Commands[CommandNames.CollisionOccurance].AddInvoker(testbutton2, "Click");

            
           

            ToolStripButton taskButton = new ToolStripButton(Properties.Resources.ToolTask.ToBitmap());
            WorkItem.UIExtensionSites[UIExtensionSiteNames.OutlookNavigation].Add(taskButton);

            WorkItem.Commands[CommandNames.TaskManagementSelected].AddInvoker(taskButton, "Click");
            

            //ToolStripButton  administrativeButton = new ToolStripButton(
            //TODO: add new items to the ToolStrip in the Shell. See the UIExtensionSites collection in the WorkItem. 
            //		See: ms-help://MS.VSCC.v80/MS.VSIPCC.v80/ms.practices.2005Nov.cab/CAB/html/03-100-Showing%20UIElements.htm
        }

        private void AddViews()
        {
            ProjectNavigation projectNavigation = WorkItem.Items.AddNew <ProjectNavigation>(ViewNames.ProjectNavigation);
            WorkItem.Workspaces[WorkspaceNames.NavigationWorkspace].Show(projectNavigation);

            TasksReview taskReview = WorkItem.Items.AddNew <TasksReview>(ViewNames.TasksReview);
            WorkItem.Workspaces[WorkspaceNames.SummaryWorkspace].Show(taskReview);

            TaskDetails taskDetails = WorkItem.Items.AddNew<TaskDetails>(ViewNames.TaskDetails);
            WorkItem.Workspaces[WorkspaceNames.DetailWorkspace].Show(taskDetails);
            //TODO: create the Module views, add them to the WorkItem and show them in 
            //		a Workspace. See: ms-help://MS.VSCC.v80/MS.VSIPCC.v80/ms.scsf.2006jun/SCSF/html/03-300-Adding%20a%20View.htm
        }

        //TODO: Add CommandHandlers and/or Event Subscriptions
        //		See: ms-help://MS.VSCC.v80/MS.VSIPCC.v80/ms.practices.2005Nov.cab/CAB/html/03-110-Registering%20Commands.htm
        //		See: ms-help://MS.VSCC.v80/MS.VSIPCC.v80/ms.practices.2005Nov.cab/CAB/html/03-080-Publishing%20and%20Subscribing%20to%20Events.htm

        [CommandHandler(CommandNames.OfflineSelection)]
        public void OnOfflineSelection(object sender, EventArgs args)
        {
            if (_isOffline)
            {
                //TODO synchronize
                WorkItem.Commands[CommandNames.ChangeOfflineImage].Execute();
                _isOffline = false;

            }
            else
            {
                IOfflineSelectionView view = WorkItem.SmartParts.AddNew<OfflineSelectionView>();
                WorkItem.Workspaces[WorkspaceNames.ModalWindows].Show(view);
                _isOffline = true;
            }
            
        }

        [CommandHandler(CommandNames.NewProject)]
        public void OnNewProject(object sender, EventArgs args)
        {
            INewProjectView view = WorkItem.SmartParts.AddNew<NewProjectView>();
            WorkItem.Workspaces[WorkspaceNames.ModalWindows].Show(view);
        }

        [CommandHandler(CommandNames.NewTask)]
        public void OnNewTask(object sender, EventArgs args)
        {
            ITaskDetailsView view = WorkItem.SmartParts.AddNew<TaskDetails>();
            view.isCreateMode = true;
            WorkItem.Workspaces[WorkspaceNames.ModalWindows].Show(view);
        }
       

        [CommandHandler(CommandNames.CollisionOccurance)]
        public void OnCollisionOccurance(object sender, EventArgs args)
        {
            ICollisionView view = WorkItem.SmartParts.AddNew<CollisionView>();
            WorkItem.Workspaces[WorkspaceNames.ModalWindows].Show(view);
        }

        [CommandHandler(CommandNames.TaskManagementSelected)]
        public void OnTaskManagementSelected(object sender, EventArgs args)
        {
            ProjectNavigation projectNavigation = WorkItem.Items.Get<ProjectNavigation>(ViewNames.ProjectNavigation);
            WorkItem.Workspaces[WorkspaceNames.NavigationWorkspace].Show(projectNavigation);

            TasksReview taskReview = WorkItem.Items.Get<TasksReview>(ViewNames.TasksReview);
            WorkItem.Workspaces[WorkspaceNames.SummaryWorkspace].Show(taskReview);

            TaskDetails taskDetails = WorkItem.Items.Get<TaskDetails>(ViewNames.TaskDetails);
            WorkItem.Workspaces[WorkspaceNames.DetailWorkspace].Show(taskDetails);
        }

        [CommandHandler(CommandNames.ChangeOfflineImage)]
        public void OnChangeOfflineImage(object sender, EventArgs args)
        {
            _offlineButton.Image = Properties.Resources.ToolWorkOffline.ToBitmap();
            _offlineButton.Text = Properties.Resources.WorkOffline;
           
        }

        [CommandHandler(CommandNames.ChangeOnlineImage)]
        public void OnChangeOnlineImage(object sender, EventArgs args)
        {
             _offlineButton.Image = Properties.Resources.ToolWorkOnline.ToBitmap();
             _offlineButton.Text = Properties.Resources.WorkOnline;
        }
    }
}
