//----------------------------------------------------------------------------------------
// patterns & practices - Smart Client Software Factory - Guidance Package
//
// This file was generated by the "Add View" recipe.
//
// A presenter calls methods of a view to update the information that the view displays. 
// The view exposes its methods through an interface definition, and the presenter contains
// a reference to the view interface. This allows you to test the presenter with different 
// implementations of a view (for example, a mock view).
//
// For more information see:
// ms-help://MS.VSCC.v80/MS.VSIPCC.v80/ms.scsf.2006jun/SCSF/html/03-030-Model%20View%20Presenter%20%20MVP%20.htm
//
// Latest version of this Guidance Package: http://go.microsoft.com/fwlink/?LinkId=62182
//----------------------------------------------------------------------------------------

using System;

using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeUI;
using Microsoft.Practices.CompositeUI.EventBroker;

using TaskViewer.TaskManagement.ServiceAgent;
using TaskViewer.TaskManagement.ServiceAgent.BusinessEntities;
using TaskViewer.Infrastructure.Interface;
using TaskViewer.TaskManagement.Constants;

namespace TaskViewer.TaskManagement
{
    public class ProjectNavigationPresenter : Presenter<IProjectNavigation>
    {
        #region fields
        private ITaskManagementServiceAgent _taskServiceAgent = null;
        #endregion 

        #region Constructors
        [InjectionConstructor]
        public ProjectNavigationPresenter
			(			
			[ServiceDependency] ITaskManagementServiceAgent taskServiceAgent
			)
        {
            _taskServiceAgent = taskServiceAgent;
        }
        #endregion

        #region Events
        [EventPublication(EventTopicNames.WorkingProjectSelected, PublicationScope.Global)]
        public event EventHandler<EventArgs<Project>> WorkingProjectSelected;

        //TODO 
        //[EventPublication(EventTopicNames.NewProjectCreated, PublicationScope.Global)]
        //public event EventHandler<EventArgs<string>> NewProjectCreated;
        #endregion

        /// <summary>
        /// This method is a placeholder that will be called by the view when it's been loaded <see cref="System.Winforms.Control.OnLoad"/>
        /// </summary>
        public override void OnViewReady()
        {
            base.OnViewReady();
            _taskServiceAgent.GetProjects(ShowProjects);
            
        }

        public void ShowProjects(Project[] projects)
        {
            View.ShowProjects(projects);
        }

        public void ProjectSelected(Project project)
        {
            if (WorkingProjectSelected != null)
            {
                WorkingProjectSelected(this, new EventArgs<Project>(project));
            }
        }

        [EventSubscription(EventTopicNames.NewProjectCreated, ThreadOption.UserInterface)]
        public void ProjectsRefreshHandler(object sender, EventArgs e)
        {
            _taskServiceAgent.GetProjects(ShowProjects);
        }
        /// <summary>
        /// Close the view
        /// </summary>
        public void OnCloseView()
        {
            base.CloseView();
        }
    }
}

