//----------------------------------------------------------------------------------------
// patterns & practices - Smart Client Software Factory - Guidance Package
//
// This file was generated by the "Add View" recipe.
//
// This class is the concrete implementation of a View in the Model-View-Presenter 
// pattern. Communication between the Presenter and this class is acheived through 
// an interface to facilitate separation and testability.
// Note that the Presenter generated by the same recipe, will automatically be created
// by CAB through [CreateNew] and bidirectional references will be added.
//
// For more information see:
// ms-help://MS.VSCC.v80/MS.VSIPCC.v80/ms.scsf.2006jun/SCSF/html/03-030-Model%20View%20Presenter%20%20MVP%20.htm
//
// Latest version of this Guidance Package: http://go.microsoft.com/fwlink/?LinkId=62182
//----------------------------------------------------------------------------------------

using System;
using System.Windows.Forms;

using Microsoft.Practices.CompositeUI.SmartParts;
using Microsoft.Practices.ObjectBuilder;
using Microsoft.Practices.CompositeUI.Utility;

using TaskViewer.Infrastructure.Interface;
using TaskViewer.TaskManagement.ServiceAgent.BusinessEntities;
using TaskViewer.Infrastructure.Interface.BusinessEntities;
using TaskViewer.Infrastructure.Library.UI;

namespace TaskViewer.TaskManagement     
{
    [SmartPart]
    public partial class TaskDetails : UserControl, ITaskDetailsView, ISmartPartInfoProvider
    {
        Task _task;
        Boolean _editState = false;
        Boolean _createMode = false;

        private void ChangeState(Boolean state)
        {
            _editState = state;
            _dueDate.Enabled = state;
            _description.Enabled = state;
            _priority.Enabled = state;
            _progress.Enabled = state;
            _status.Enabled = state;
            _summary.Enabled = state;
            _assignedTo.Enabled = state;
            _save.Visible = state;
            if(state)
            {
                _edit.Text = Properties.Resources.btnCancelText;
            }
            else
            {
                _edit.Text = Properties.Resources.btnEditText;
            }

        }
        public TaskDetails()
        {
            InitializeComponent();            
           ((ITaskDetailsView)this).ResetViewToEmpty();
           
        }

        

        /// <summary>
        /// Sets the presenter. The dependency injection system will automatically
        /// create a new presenter for you.
        /// </summary>
        [CreateNew]
        public TaskDetailsPresenter Presenter
        {
            set
            {
                Guard.ArgumentNotNull(value, "Presenter");
                _presenter = value;
                _presenter.View = this;
            }
        }

        protected override void OnLoad(EventArgs e)
        {
            _presenter.OnViewReady();
        }

        private void label1_Click(object sender, EventArgs e)
        {

        }



        #region ITaskDetailsView Members
        private delegate void ResetViewToEmptyDelegate();

        public void ResetViewToEmpty()
        {
            if (InvokeRequired)
            {
                BeginInvoke(new ResetViewToEmptyDelegate(InnerResetViewToEmpty));
            }
            else
            {
                InnerResetViewToEmpty();
            }
        }



        private void InnerResetViewToEmpty()
        {
            _task = null;
            _coverPanel.BringToFront();
        }

        private delegate void ShowTaskDelegate( Task task);
        public void ShowTask(Task task)
        {

            Guard.ArgumentNotNull(task, "task");
            if (InvokeRequired)
            {
                BeginInvoke(new ShowTaskDelegate(InnerShowTask));
            }
            else
            {
                InnerShowTask(task);
            }

        }

        private void InnerShowTask(Task task)
        {
            _task = task;
            _coverPanel.Hide();
            ChangeState(false);
            _taskNumber.Text = _task.TaskId.ToString();
            _dateModified.Text = _task.DateModified.ToShortDateString();
            _modifiedBy.Text = _task.ModifiedBy;
            _dueDate.Value = _task.DueDate;
            _summary .Text= _task.TaskSummary;
            _description.Text = _task.TaskDescription;
            _assignedTo.SelectedValue = _task.AssignedToID;
            _priority.SelectedValue = _task.PriorityID;
            _status.SelectedValue = _task.StatusID;
            _progress.Text = _task.Progress.ToString();
            
        }

        private Task UpdateTask()
        {
            Task task = new Task(
                _task.TaskId,
                _task.ProjectId,
                _task.ModifiedByID, //TODO add it from current user object
                (int)_assignedTo.SelectedValue,
                _summary.Text,
                _description.Text,
                (int)_priority.SelectedValue,
                (int)_status.SelectedValue,
                Convert.ToInt32(_progress.Text),
                _task.IsDeleted,
                _dueDate.Value,
                DateTime.Now,
                _task.DateCreated,
                _priority.SelectedText,
                _status.SelectedText,
                _assignedTo.SelectedText,
                _task.ModifiedBy);//TODO

            return task;


        }

        

        #endregion

        #region ITaskDetailsView Members


        public void ShowPriorities(Priority[] priorities)
        {
            //TODO check if invoke required, do i need it and in which cases its needed
            _priorityBindingSource.DataSource = priorities;
        }

        public void ShowStatus(Status[] statuses)
        {
            _statusBindingSource.DataSource = statuses;
        }

        public void ShowAssignedTo(User[] users)
        {
            _assignedToBindingSource.DataSource = users;
        }

        #endregion

        private void _edit_Click(object sender, EventArgs e)
        {
            ChangeState(!_editState);
        }

        private void _save_Click(object sender, EventArgs e)
        {
            ChangeState(!_editState);
            _presenter.UpdateTask(this.UpdateTask());

        }

        #region ITaskDetailsView Members


        public bool isCreateMode
        {
            get
            {
                return _createMode;
            }
            set
            {
                _createMode = value;
                if (value)
                {
                    _coverPanel.Hide();
                    _editState = true;
                }
            }
        }

        #endregion

        #region ISmartPartInfoProvider Members

        public ISmartPartInfo GetSmartPartInfo(Type smartPartInfoType)
        {
            ISmartPartInfo spi;


            if (smartPartInfoType.IsAssignableFrom(typeof(WindowSmartPartInfo)))
            {
                WindowSmartPartInfo wspi = new WindowSmartPartInfo();
                wspi.Modal = true;
                wspi.MinimizeBox = false;
                wspi.MaximizeBox = false;
                wspi.ControlBox = true;

                wspi.Keys[WindowWorkspaceSetting.FormBorderStyle] = FormBorderStyle.FixedDialog;
                wspi.Keys[WindowWorkspaceSetting.AcceptButton] = _save;
                wspi.Keys[WindowWorkspaceSetting.CancelButton] = _edit;
                spi = wspi;
            }
            else
            {
                spi = Activator.CreateInstance(smartPartInfoType) as ISmartPartInfo;
            }

            spi.Description = Properties.Resources.NewTaskTitle;
            spi.Title = Properties.Resources.NewTaskDescription;

            return spi;
        }

        #endregion
    }
}

