﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using TaskViewer.AuthService.BusinessEntities;
using TaskViewer.AuthService.DataAccess.Generic;
using System.Globalization;

namespace TaskViewer.AuthService.DataAccess
{
    /// <summary>
    /// This class is responsible for turning a selection criteria
    /// into a select statement and database query parameters.
    /// </summary>
    internal class AuthenticateUserSelectionFactory : ISelectionFactory<AuthenticateUserIdentity>
    {
        /// <summary>
        /// Creates the AuthenticateUserSelectionFactory to build a select statement for
        /// the given AuthenticateUserIdentity.
        /// </summary>
        /// <param name="AuthenticateUserIdentity">AuthenticateUserIdentity to select from the database.</param>      
        public AuthenticateUserSelectionFactory()
        {
        }

		#region ISelectionFactory<AuthenticateUserIdentity> Members
		public DbCommand ConstructSelectCommand(Database db, AuthenticateUserIdentity authenticateUserIdentity)
		{
 		
            DbCommand command = db.GetStoredProcCommand("dbo.AuthenticateUser");
			db.AddInParameter(command, "Password", DbType.String, authenticateUserIdentity.Password);
			db.AddInParameter(command, "UserName", DbType.String, authenticateUserIdentity.UserName);
			return command;
        }
        #endregion
        
        #region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
				case "Password":
					return "Password";
				case "UserName":
					return "UserName";
				default:
					throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
			}
        }
        #endregion  
    }
}

