﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using TaskViewer.AuthService.BusinessEntities;
using TaskViewer.AuthService.DataAccess.Generic;
using System.Globalization;

namespace TaskViewer.AuthService.DataAccess
{
    /// <summary>
    /// A factory object that is responsible for taking a User
    /// and generating the corresponding SQL to insert that
    /// User into the database. It also includes a method
    /// to grab the returned ID from the call and updating the
    /// User object with it.
    /// </summary>
    internal class UserInsertFactory : IDbToBusinessEntityNameMapper, IInsertFactory<User>
    {
        /// <summary>
        /// Creates the UserInsertFactory to build an insert statement for
        /// the given User object.
        /// </summary>
        /// <param name="User">New User to insert into the database.</param>
        public UserInsertFactory()
        {
        }

		#region IInsertFactory<User> Members
		
        public DbCommand ConstructInsertCommand(Database db, User user)
        {            
			DbCommand command = db.GetStoredProcCommand("dbo.InsertUser");
			
				db.AddInParameter(command, "IsAccountLocked", DbType.Boolean, user.IsAccountLocked);
				db.AddInParameter(command, "IsAdministrator", DbType.Boolean, user.IsAdministrator);
			if(user.UserEmail != null)
			{
				db.AddInParameter(command, "UserEmail", DbType.String, user.UserEmail);
			}
			if(user.UserFullName != null)
			{
				db.AddInParameter(command, "UserFullName", DbType.String, user.UserFullName);
			}
			if(user.UserName != null)
			{
				db.AddInParameter(command, "UserName", DbType.String, user.UserName);
			}
			return command;
        }

        public void SetNewID(Database db, DbCommand command, User user)
        {
            //TODO: Provide set mapping
        }
        
        #endregion
        
		#region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
				case "IsAccountLocked":
					return "IsAccountLocked";
				case "IsAdministrator":
					return "IsAdministrator";
				case "UserEmail":
					return "UserEmail";
				case "UserFullName":
					return "UserFullName";
				case "UserName":
					return "UserName";
				default:
					throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
			}
        }
        #endregion        
    }

}

