﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using TaskViewer.AuthService.BusinessEntities;
using System.Data.SqlClient;
using System.Diagnostics;
using TaskViewer.AuthService.DataAccess.SQLServer;

namespace TaskViewer.AuthService.DataAccess
{
    /// <summary>
    /// Respository that lets you find User in the
    /// CRM database.
    /// </summary>
    public class UserRepository : Repository<User>
    {
        private string databaseName;

        public UserRepository(string databaseName)
            : base(databaseName)
        {
            this.databaseName = databaseName;
        }
        

		public User AuthenticateUser(System.String password, System.String userName)
        {
            ISelectionFactory<AuthenticateUserIdentity> selectionFactory = new AuthenticateUserSelectionFactory();
            
            try
            {

				AuthenticateUserIdentity authenticateUserIdentity = new AuthenticateUserIdentity(password, userName);
				return base.FindOne(selectionFactory, new AuthenticateUserFactory(), authenticateUserIdentity);
			}
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }
            
			return new User();
        }

		public List<User> GetUsers()
        {
            ISelectionFactory<NullableIdentity> selectionFactory = new GetUsersSelectionFactory();
            
            try
            {
				NullableIdentity nullableIdentity = new NullableIdentity();
				return base.Find(selectionFactory, new GetUsersFactory(), nullableIdentity);
			}
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }
            
            return new List<User>();
        }

		public User GetUserInfo(System.Int32 userID)
        {
            ISelectionFactory<System.Int32> selectionFactory = new GetUserInfoSelectionFactory();
            
            try
            {
				return base.FindOne(selectionFactory, new GetUserInfoFactory(), userID);
			}
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }
            
			return new User();
        }

        public void Add(User user)
        {
            UserInsertFactory insertFactory = new UserInsertFactory();
            try
            {
                base.Add(insertFactory, user);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, insertFactory);
            }
        }

        public void Remove(System.Int32 userID)
        {
			IDeleteFactory<System.Int32> deleteFactory = new UserDeleteFactory();

            try
            {
				base.Remove(deleteFactory, userID);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, deleteFactory);
            }            
        }
      

		public void Save(User user)
        {
            UserUpdateFactory updateFactory = new UserUpdateFactory();
            try
            {
                base.Save(updateFactory, user);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, updateFactory);
            }
        }

        private void HandleSqlException(SqlException ex, IDbToBusinessEntityNameMapper mapper)
        {
            if(ex.Number == ErrorCodes.SqlUserRaisedError)
            {
                switch(ex.State)
                {
                    case ErrorCodes.ValidationError:
                        string[] messageParts = ex.Errors[0].Message.Split(':');
                        throw new RepositoryValidationException(
                            mapper.MapDbParameterToBusinessEntityProperty(messageParts[0]),
                            messageParts[1], ex);

                    case ErrorCodes.ConcurrencyViolationError:
                        throw new ConcurrencyViolationException(ex.Message, ex);

                }
            }

            throw new RepositoryFailureException(ex);
        }
    }
}

