﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using TaskViewer.AuthService.BusinessEntities;
using TaskViewer.AuthService.DataAccess.Generic;
using System.Globalization;

namespace TaskViewer.AuthService.DataAccess
{
    /// <summary>
    /// This factory class takes a User object and generates
    /// the dbcommand object needed to call the underlying update
    /// stored proc.
    /// </summary>
    internal class UserUpdateFactory : IDbToBusinessEntityNameMapper, IUpdateFactory<User>
    {
        /// <summary>
        /// Creates the UserUpdateFactory to build an update statement for
        /// the given User object.
        /// </summary>
        /// <param name="User">User to update into the database.</param>
        public UserUpdateFactory()
        {
        }
        
        #region IUpdateFactory<User> Members

        public DbCommand ConstructUpdateCommand(Database db, User user)
        {
			DbCommand command = db.GetStoredProcCommand("dbo.UpdateUser");
			
			db.AddInParameter(command, "IsAccountLocked", DbType.Boolean, user.IsAccountLocked);
			db.AddInParameter(command, "IsAdministrator", DbType.Boolean, user.IsAdministrator);
			if(user.UserEmail != null)
			{
				db.AddInParameter(command, "UserEmail", DbType.String, user.UserEmail);
			}
			if(user.UserFullName != null)
			{
				db.AddInParameter(command, "UserFullName", DbType.String, user.UserFullName);
			}
			db.AddInParameter(command, "UserID", DbType.Int32, user.UserID);
			return command;
        }
        #endregion
        
		#region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
				case "IsAccountLocked":
					return "IsAccountLocked";
				case "IsAdministrator":
					return "IsAdministrator";
				case "UserEmail":
					return "UserEmail";
				case "UserFullName":
					return "UserFullName";
				case "UserID":
					return "UserID";
				default:
					throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
			}
        }
        #endregion        
    }
}

