﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using TaskViewer.ProjectManagement.BusinessEntities;
using TaskViewer.DataService.DataAccess.Generic;
using System.Globalization;

namespace TaskViewer.DataService.DataAccess
{
    /// <summary>
    /// A factory object that is responsible for taking a Project
    /// and generating the corresponding SQL to insert that
    /// Project into the database. It also includes a method
    /// to grab the returned ID from the call and updating the
    /// Project object with it.
    /// </summary>
    internal class ProjectInsertFactory : IDbToBusinessEntityNameMapper, IInsertFactory<Project>
    {
        /// <summary>
        /// Creates the ProjectInsertFactory to build an insert statement for
        /// the given Project object.
        /// </summary>
        /// <param name="Project">New Project to insert into the database.</param>
        public ProjectInsertFactory()
        {
        }

        #region IInsertFactory<Project> Members

        public DbCommand ConstructInsertCommand(Database db, Project project)
        {
            DbCommand command = db.GetStoredProcCommand("dbo.InsertProject");

            if (project.ProjectDescription != null)
            {
                db.AddInParameter(command, "ProjectDescription", DbType.String, project.ProjectDescription);
            }
            if (project.ProjectName != null)
            {
                db.AddInParameter(command, "ProjectName", DbType.String, project.ProjectName);
            }
            return command;
        }

        public void SetNewID(Database db, DbCommand command, Project project)
        {
            //TODO: Provide set mapping
        }

        #endregion

        #region IDbToBusinessEntityNameMapper Members
        /// <summary>
        /// Maps a field name in the database (usually a parameter name for a stored proc)
        /// to the corresponding business entity property name.
        /// </summary>
        /// <remarks>This method is intended for error message handling, not for reflection.</remarks>
        /// <param name="dbParameter">Name of field/parameter that the database knows about.</param>
        /// <returns>Corresponding business entity field name.</returns>
        public string MapDbParameterToBusinessEntityProperty(string dbParameter)
        {
            switch (dbParameter)
            {
                case "ProjectDescription":
                    return "ProjectDescription";
                case "ProjectName":
                    return "ProjectName";
                default:
                    throw new ArgumentException(string.Format(CultureInfo.CurrentCulture, GenericResources.InvalidParameterName), dbParameter);
            }
        }
        #endregion
    }

}

