﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using TaskViewer.ProjectManagement.BusinessEntities;
using System.Data.SqlClient;
using System.Diagnostics;
using TaskViewer.DataService.DataAccess.SQLServer;

namespace TaskViewer.DataService.DataAccess
{
    /// <summary>
    /// Respository that lets you find SyncTask in the
    /// CRM database.
    /// </summary>
    public class SyncTaskRepository : Repository<SyncTask>
    {
        private string databaseName;

        public SyncTaskRepository(string databaseName)
            : base(databaseName)
        {
            this.databaseName = databaseName;
        }
        

        public void Add(SyncTask syncTask)
        {
            SyncTaskInsertFactory insertFactory = new SyncTaskInsertFactory();
            try
            {
                base.Add(insertFactory, syncTask);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, insertFactory);
            }
        }
      

		public void Save(SyncTask syncTask)
        {
            SyncTaskUpdateFactory updateFactory = new SyncTaskUpdateFactory();
            try
            {
                base.Save(updateFactory, syncTask);
            }
            catch (SqlException ex)
            {
                HandleSqlException(ex, updateFactory);
            }
        }

        private void HandleSqlException(SqlException ex, IDbToBusinessEntityNameMapper mapper)
        {
            if(ex.Number == ErrorCodes.SqlUserRaisedError)
            {
                switch(ex.State)
                {
                    case ErrorCodes.ValidationError:
                        string[] messageParts = ex.Errors[0].Message.Split(':');
                        throw new RepositoryValidationException(
                            mapper.MapDbParameterToBusinessEntityProperty(messageParts[0]),
                            messageParts[1], ex);

                    case ErrorCodes.ConcurrencyViolationError:
                        throw new ConcurrencyViolationException(ex.Message, ex);

                }
            }

            throw new RepositoryFailureException(ex);
        }
    }
}

