﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Text;
using Microsoft.Practices.EnterpriseLibrary.Data;
using TaskViewer.ProjectManagement.BusinessEntities;
using System.Data.SqlClient;
using System.Diagnostics;
using TaskViewer.DataService.DataAccess.SQLServer;

namespace TaskViewer.DataService.DataAccess
{
    /// <summary>
    /// Respository that lets you find Task in the
    /// CRM database.
    /// </summary>
    public class TaskRepository : Repository<Task>
    {
        private string databaseName;

        public TaskRepository(string databaseName)
            : base(databaseName)
        {
            this.databaseName = databaseName;
        }
        

        public List<Task> GetTasks(System.Int32 projectID)
        {
            ISelectionFactory<System.Int32> selectionFactory = new GetTasksSelectionFactory();
            
            try
            {
				return base.Find(selectionFactory, new GetTasksFactory(), projectID);
			}
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }
            
            return new List<Task>();
        }

		public Task GetOneTask(System.Int32 taskID)
        {
            ISelectionFactory<System.Int32> selectionFactory = new GetOneTaskSelectionFactory();
            
            try
            {
				return base.FindOne(selectionFactory, new GetOneTaskFactory(), taskID);
			}
            catch (SqlException ex)
            {
                HandleSqlException(ex, selectionFactory);
            }
            
			return new Task();
        }

        private void HandleSqlException(SqlException ex, IDbToBusinessEntityNameMapper mapper)
        {
            if(ex.Number == ErrorCodes.SqlUserRaisedError)
            {
                switch(ex.State)
                {
                    case ErrorCodes.ValidationError:
                        string[] messageParts = ex.Errors[0].Message.Split(':');
                        throw new RepositoryValidationException(
                            mapper.MapDbParameterToBusinessEntityProperty(messageParts[0]),
                            messageParts[1], ex);

                    case ErrorCodes.ConcurrencyViolationError:
                        throw new ConcurrencyViolationException(ex.Message, ex);

                }
            }

            throw new RepositoryFailureException(ex);
        }
    }
}

