using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;

using TaskViewer.DataService.ServiceContracts;
using TaskViewer.ProjectManagement.BusinessEntities;
using TaskViewer.DataService.DataContracts;
using TaskViewer.DataService.BusinessLogic;


namespace TaskViewer.DataService.ServiceImplementation
{
    [ServiceBehavior(Name = "DataService", Namespace = "http://TaskViewer.DataService.ServiceContracts/2007/07")]
    public class DataService : IDataService
    {
        #region IDataService Members

        
        public GetProjectsResponse GetProjects(GetProjectsRequest request)
        {
            Authenticate(request.ticket);
            GetProjectsResponse response = new GetProjectsResponse();
            DataContracts.ProjectsInfo projectsInfo = new ProjectsInfo();
            ProjectManager manager = new ProjectManager();
            List<ProjectsInfo.ProjectsRow> projects = manager.GetProjectList().ConvertAll<ProjectsInfo.ProjectsRow>(
                new Converter<Project, ProjectsInfo.ProjectsRow>(
                ProjectsInfoAndProjectTranslator.TranslateProjectToProjectsInfo));
            foreach (ProjectsInfo.ProjectsRow row in projects)
            {
                projectsInfo.Projects.ImportRow(row);
            }
            response.GetProjectsResult = projectsInfo;
            return response;
        }

        public GetTasksResponse GetTasks(GetTasksRequest request)
        {
            Authenticate(request.ticket);
            GetTasksResponse response = new GetTasksResponse();
            DataContracts.TasksInfo taskInfo = new TasksInfo();
            TaskManager manager = new TaskManager();
            List<TasksInfo.TasksRow> tasks = manager.GetTaskList(request.projectID).ConvertAll<TasksInfo.TasksRow>(
                new Converter<Task,TasksInfo.TasksRow>(
                TasksInfoAndTaskTranslator.TranslateTaskToTasksInfo));
            foreach (TasksInfo.TasksRow row in tasks)
            {
                taskInfo.Tasks.ImportRow(row);
            }
            response.GetTasksResult = taskInfo;
            return response;
        }

        public UpdateTaskResponse UpdateTasks(UpdateTaskRequest request)
        {
            Authenticate(request.ticket);
            throw new Exception("The method or operation is not implemented.");
        }

        
        public GetLookUpTablesResponse GetLookupTables(GetLookUpTablesRequest request)
        {
            Authenticate(request.ticket);
            GetLookUpTablesResponse lookUpTables= new GetLookUpTablesResponse();

            DataContracts.LookupInfo lookUpDataSet = new TaskViewer.DataService.DataContracts.LookupInfo();
            LookUpManager manager = new LookUpManager();
            List<LookupInfo.PrioritiesRow> priorities= manager.GetPriorityList().ConvertAll<LookupInfo.PrioritiesRow>(
                new Converter<Priority, LookupInfo.PrioritiesRow>(
                LookupInfoAndPriorityTranslator.TranslatePriorityToLookupInfo));
            List<LookupInfo.StatusesRow> statuses = manager.GetStatusList().ConvertAll<LookupInfo.StatusesRow>(
                new Converter<Status, LookupInfo.StatusesRow>(
                LookupInfoAndStatusTranslator.TranslateStatusToLookupInfo));
            List<LookupInfo.UsersRow> users = manager.GetUserList().ConvertAll<LookupInfo.UsersRow>(
                new Converter<User, LookupInfo.UsersRow>(
                LookupInfoAndUserTranslator.TranslateUserToLookupInfo));

            foreach(LookupInfo.PrioritiesRow priorityRow in priorities)
            {
                lookUpDataSet.Priorities.ImportRow(priorityRow);
            }
            foreach(LookupInfo.StatusesRow statusRow in statuses)
            {
                lookUpDataSet.Statuses.ImportRow(statusRow);
            }
            foreach (LookupInfo.UsersRow userRow in users)
            {
                lookUpDataSet.Users.ImportRow(userRow);
            }

            lookUpTables.getLookupTablesResponse = lookUpDataSet;
            return lookUpTables;

            
            
        }

        public GetProjectHistoryResponse GetProjectHistory(GetProjectHistoryRequest request)
        {
            Authenticate(request.ticket);
            throw new NotImplementedException();
        }

        public InsertProjectResponse InsertProject(InsertProjectRequest request)
        {
            Authenticate(request.ticket);
            ProjectManager manager = new ProjectManager();
            InsertProjectResponse response = new InsertProjectResponse();
            try
            {
                manager.AddProject(request.projectName, request.projectDescription);
                response.insertProjectResult = 0;
            }
            catch
            {
                response.insertProjectResult = -1;
            }
            return response;
            
        }

        private bool Authenticate(string ticket)
        {
            //TODO: if there is enough time
            return true;
        }
        #endregion
    }
}
